# Optional Modules

After executing `npx @ackee/create-react-app my-app` command, you will be prompt if you want to include optional modules in your project.

By default, you choose to include/exclude the `auth` and `form` modules. Note that if you choose to include `auth` module, the `form` module is included automatically.

---

## How to add a new optional module?

Let's say I want to add a new optional module called `users`. Where to start?

1. Add new prompt to `packages/create-react-app/custom/scripts/templateModulePrompts/index.js`

   - Extend `OptionalModuleName` with `users-module`.
   - Set initial value to `result` if module is optional.
   - Add the prompt, e.g.:
     ```js
     result[OptionalModuleName.USERS] = await modulePrompt({
       name: OptionalModuleName.USERS,
       initial: true,
     });
     ```

   Now you call `npx cra-local` to check out you will see this new prompt.

2. Add module's dependencies in `packages/react-scripts/custom/scripts/init/optional-modules/modules`

   - Copy/paste some existing `get...ModuleDependencies.js` file and create `getUsersModuleDependencies.js`.
   - In `getUsersModuleDependencies` you specify which files, directories, or dependencies from package.json will be removed in the case when the module isn't included.
     > Note that `files` must include all files that are using the conditional comments (e.g. `@use-auth-module-start` / `@use-auth-module-end`)

3. Initialize a new optional module in `packages/react-scripts/custom/scripts/init/optional-modules/applyResolutions.js`:

   ```js
   await processOptionalModuleHooks(
     new OptionalModule({
       name: OptionalModuleName.USERS,
       useModule: result[OptionalModuleName.USERS],
       dependencies: getUsersModuleDependencies(resolveSrc),
       prettierFormatter,
       appPackage,
       templatePackage,
     })
   );
   ```

4. Optionally, use conditional comments in your skeleton to mark module-specific code. Available comments for this module are:

   ```js
   // @use-users-module-begin
   function foo() {
     console.log('I will be in project only if users module is avail.');
   }
   // @use-users-module-end
   ```

   Again, good idea is to check changes locally with `npx cra-local` before releasing a new `create-react-app`, `react-scripts` and `cra-template-typescript` packages.
