'use strict';

const path = require('path');

const getAuthModuleDependencies = require('./modules/getAuthModuleDependencies');
const getFormModuleDependencies = require('./modules/getFormModuleDependencies');
const getAntdModuleDependencies = require('./modules/getAntdModuleDependencies');

const createPrettierFormatter = require('./utils/createPrettierFormatter');
const OptionalModule = require('./OptionalModule/OptionalModule');
const processOptionalModuleHooks = require('./processOptionalModuleHooks');

async function applyOptionalModulesResolutions(
  { OptionalModuleName, result },
  { appPackage, appPath, templatePackage }
) {
  const resolveSrc = file => path.resolve(appPath, 'src', file);
  const prettierFormatter = await createPrettierFormatter(
    path.resolve(appPath, 'prettier.config.js')
  );

  await processOptionalModuleHooks(
    new OptionalModule({
      name: OptionalModuleName.AUTH,
      useModule: result[OptionalModuleName.AUTH],
      dependencies: getAuthModuleDependencies(resolveSrc),
      prettierFormatter,
      appPackage,
      templatePackage,
    })
  );

  await processOptionalModuleHooks(
    new OptionalModule({
      name: OptionalModuleName.FORM,
      useModule: result[OptionalModuleName.FORM],
      dependencies: getFormModuleDependencies(resolveSrc),
      prettierFormatter,
      appPackage,
      templatePackage,
    })
  );

  await processOptionalModuleHooks(
    new OptionalModule({
      name: OptionalModuleName.ANTD,
      useModule: result[OptionalModuleName.ANTD],
      dependencies: getAntdModuleDependencies(resolveSrc),
      prettierFormatter,
      appPackage,
      templatePackage,
    })
  );
}

module.exports = applyOptionalModulesResolutions;
