'use strict';

const getExistingFiles = require('./getExistingFiles');
const chalk = require('chalk');

/**
 * - resolve relative paths
 * - filter out existing & non-existing paths
 *
 * @param {(path: string) => string} pathResolver
 * @param {Object} moduleDependencies
 * @param {string[]} [moduleDependencies.files]
 * @param {string[]} [moduleDependencies.directories]
 * @param {Object} [moduleDependencies.packageJson]
 * @param {string[]}  [moduleDependencies.packageJson.dependencies]
 */
function getModuleDependenciesFactory(moduleDependencies) {
  /**
   * @param {string} pathResolver
   */
  return async pathResolver => {
    const files = await getExistingFiles(
      (moduleDependencies.files || []).map(pathResolver)
    );
    const directories = await getExistingFiles(
      (moduleDependencies.directories || []).map(pathResolver)
    );

    if (files.nonExistingPaths.length > 0) {
      console.log(
        chalk.yellow(
          `[optional-modules]: Provided files as module's dependencies don't exist:`
        )
      );
      console.log(chalk.yellow(files.nonExistingPaths.join('\n')));
    }

    if (directories.nonExistingPaths.length > 0) {
      console.log(
        chalk.yellow(
          `[optional-modules]: Provided directories as module's dependencies don't exist:`
        )
      );
      console.log(chalk.yellow(directories.nonExistingPaths.join('\n')));
    }

    return {
      files: files.existingPaths,
      directories: directories.existingPaths,
      packageJson: moduleDependencies.packageJson || {},
    };
  };
}

module.exports = getModuleDependenciesFactory;
